package com.example.catchTheLetters.utils;

import io.minio.*;
import io.minio.messages.Item;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.MediaType;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * @description minio工具类
 * @className MinioUtil
 * @auther spyn
 */

@Slf4j
@Data
@Configuration
@ConfigurationProperties(prefix = "minio")
public class MinioUtil {

    private String url;

    private String accessKey;

    private String secretKey;

    private String bucketName;

    private MinioClient minioClient;

    @Bean
    public MinioClient getMinioClient() {
        try {
            minioClient = MinioClient.builder()
                    .endpoint(url)
                    .credentials(accessKey, secretKey)
                    .build();
            // 如果没有，创建默认桶
            boolean isExist = checkBucketExist(bucketName);
            if (!isExist) {
                boolean isTrue = createBucket(bucketName);
                if (isTrue) {
                    log.info("默认桶创建成功");
                } else {
                    log.error("默认桶创建失败");
                    throw new RuntimeException("默认桶创建失败");
                }
            }
        } catch (Exception e) {
            log.error("初始化minioClient失败", e);
        }
        return minioClient;
    }

    /**
     * 创建桶
     * @param bucketName 桶名
     * @return 是否创建成功
     */
    public Boolean createBucket(String bucketName) {
        if (!StringUtils.hasLength(bucketName)) {
            log.error("桶名不能为空");
            return false;
        }
        try {
            boolean isExist = minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
            if (!isExist) {
                minioClient.makeBucket(MakeBucketArgs.builder().bucket(bucketName).build());
                log.info("桶创建成功");
                return true;
            } else {
                log.info("桶已存在");
                return false;
            }
        } catch (Exception e) {
            log.error("创建bucket失败", e);
            return false;
        }
    }

    /**
     * 检测桶是否存在
     * @param bucketName 桶名
     * @return 是否存在
     */
    public Boolean checkBucketExist(String bucketName) {
        if (!StringUtils.hasLength(bucketName)) {
            log.error("桶名不能为空");
            return false;
        }

        try {
            return minioClient.bucketExists(BucketExistsArgs.builder().bucket(bucketName).build());
        } catch (Exception e) {
            log.error("检测bucket失败", e);
            return false;
        }
    }

    /**
     * 检测是否存在某个文件
     * @param bucketName 桶名
     * @param path 文件路径
     * @return 是否存在
     */
    public Boolean checkFileExist(String bucketName, String path) {
        if (!StringUtils.hasLength(bucketName) || !StringUtils.hasLength(path)) {
            log.error("桶名和文件路径不能为空");
            return false;
        }

        try {
            return (checkBucketExist(bucketName) &&
                    minioClient.statObject(StatObjectArgs.builder().bucket(bucketName).object(path).build()) != null);
        } catch (Exception e) {
            return false;
        }
    }

    /**
     * 删除文件
     * @param bucketName 桶名
     * @param path 文件路径
     * @param isDeep 是否深度删除
     * @return 是否删除成功
     */
    public Boolean deleteFile(String bucketName, String path, Boolean isDeep) {
        if (!StringUtils.hasLength(bucketName) || !StringUtils.hasLength(path)) {
            log.error("桶名和文件路径不能为空");
            return false;
        }

        try {
            ListObjectsArgs args = ListObjectsArgs.builder().bucket(bucketName).prefix(path).recursive(isDeep).build();
            Iterable<Result<Item>> listObjects = minioClient.listObjects(args);
            listObjects.forEach(objectResult -> {
                try {
                    Item item = objectResult.get();
                    minioClient.removeObject(RemoveObjectArgs.builder().bucket(bucketName).object(item.objectName()).build());
                } catch (Exception e) {
                    log.info("删除文件夹中的文件异常", e);
                    throw new RuntimeException("删除文件夹中的文件异常");
                }
            });
            return true;
        } catch (Exception e) {
            log.info("删除失败", e);
            return false;
        }
    }

    /**
     * 删除文件
     * @param path 文件路径
     * @return 是否删除成功
     */
    public Boolean deleteFile(String path) {
        return deleteFile(bucketName, path, true);
    }

    /**
     * 上传文件
     * @param bucketName 桶名
     * @param file 文件
     * @param path 路径
     * @return 是否上传成功
     */
    public Boolean uploadFile(String bucketName, MultipartFile file, String path) {
        if (!StringUtils.hasLength(bucketName) || file == null || !StringUtils.hasLength(path)) {
            log.error("桶名、文件和路径不能为空");
            return false;
        }

        try {
            // 检查是否有同名文件
            if (checkFileExist(bucketName, path)) {
                log.error("已存在同名文件");
                return false;
            }
            // 资源的媒体类型
            String contentType = MediaType.APPLICATION_OCTET_STREAM_VALUE;//默认未知二进制流
            InputStream inputStream = file.getInputStream();
            // 将文件上传到MinIO服务器
            minioClient.putObject(
                    PutObjectArgs.builder()
                            .bucket(bucketName)
                            .object(path)
                            .stream(inputStream, file.getSize(), -1)
                            .contentType(contentType)
                            .build()
            );
            inputStream.close();
            return true;
        } catch (Exception e) {
            log.error("上传文件失败", e);
            return false;
        }
    }

    /**
     * 上传文件
     * @param file 文件
     * @param path 路径
     * @return 是否上传成功
     */
    public Boolean uploadFile(MultipartFile file, String path) {
        return uploadFile(bucketName, file, path);
    }

    /**
     * 上传文件
     * @param file 文件
     * @return 是否上传成功
     */
    public Boolean uploadFile(MultipartFile file) {
        return uploadFile(bucketName, file, file.getOriginalFilename());
    }

    /**
     * 查看桶内目录树
     * @param bucketName 桶名
     * @return 目录树
     */
    public List<String> fileList(String bucketName) {
        if (!StringUtils.hasLength(bucketName)) {
            log.error("桶名不能为空");
            return null;
        }

        try {
            List<String> list = new ArrayList<>();
            Iterable<Result<Item>> results = minioClient.listObjects(ListObjectsArgs.builder().bucket(bucketName).recursive(true).build());
            results.forEach(itemResult -> {
                try {
                    Item item = itemResult.get();
                    list.add(item.objectName());
                } catch (Exception e) {
                    log.error("获取文件列表失败", e);
                }
            });
            return list;
        } catch (Exception e) {
            log.error("获取文件列表失败", e);
            return null;
        }
    }

    /**
     * 查看桶内目录树
     * @return 目录树
     */
    public List<String> fileList() {
        return fileList(bucketName);
    }
}
